import type { Obj, XhrConfig, XhrGet } from './index';
import Deferred from './Deferred';
export default class Xhr<T> {
    static parseResult(xhr: XMLHttpRequest): Response;
    static extractQueryParams(url: string): Obj;
    onError: (e: ProgressEvent) => void;
    onStart: (e: ProgressEvent) => void;
    onAbort: (e: ProgressEvent) => void;
    onProgress: (e: ProgressEvent) => void;
    onEnd: (result: any | null, e: ProgressEvent) => void;
    static onBeforeSendList: Array<(params: Obj) => Obj>;
    static onBeforeSend(cb: (params: Obj) => Obj): void;
    token: string | null;
    url: string;
    params: Obj;
    sendAs: 'multipart' | 'json';
    timeout: number;
    port: number;
    responseType: 'arraybuffer' | 'blob' | 'json' | 'text';
    isPending: boolean;
    _eventsReady: boolean;
    _oXHR: XMLHttpRequest;
    _onEnd: (e: ProgressEvent) => void;
    _onError: (e: ProgressEvent) => void;
    _deferred: Deferred<T>;
    _isXhrResolved: boolean;
    _isXhrRejected: boolean;
    _eventReady: boolean;
    constructor(xhrParams?: XhrConfig, params?: Obj);
    static new<Z>(paramsObj?: XhrConfig): Xhr<Z>;
    _getUrl(initial?: Obj): string;
    removeEvents(): void;
    post(paramsObj: XhrConfig): Promise<T>;
    put(paramsObj: XhrConfig): Promise<T>;
    /**
     * Rewrite URL or send query parameters
     * Return a Promise with a function to abort the xhr
     *
     * @returns {Promise}, consolidate the promise with the `abortXhr` function
     */
    get(paramsObj?: XhrConfig): XhrGet<T>;
    delete(paramsObj: XhrConfig): Promise<any>;
    /**
     * Abort xhr query and reject promise
     */
    abort(): Promise<T>;
    static stringifyUrl(url: string, params?: Obj): string;
    /**
     * return FormData
     */
    static getFormData(data: Obj): FormData;
    /**
     * Force to resolve deferred
     * @param res
     */
    resolve(res: T): Promise<T>;
    /**
     * Force to reject deferred
     * @param res
     */
    reject(res: Error): Promise<any>;
    static _stringifyForPathParam(decodedUrl: string, attrName: string, obj: any): string;
    /**
     * url with path params will be replaced by params values
     */
    static injectParamsInUrl(url: string, params?: Obj | Array<unknown>): {
        url: string;
        params: Obj;
    };
    /**
     * Set events and bind params
     * @param {XMLHttpRequest} xhr
     * @private
     */
    _setEvents(xhr: XMLHttpRequest): void;
    /**
     * Send xhr
     * If params contain files, use multipart, else JSON
     */
    _send(): void;
    /**
     * The constructor can be called by the `new` or by each public method
     */
    _constructor(paramsObj: XhrConfig, params?: Obj): void;
}
